<?php
/*
Plugin Name: Influential Commenters
Plugin URI: http://www.graemeboy.com/influential-commenters
Description: Lists your commenters by popularity of website
Version: 1
Author: Graeme Boy
Author URI: http://www.graemeboy.com
*/

add_action('admin_menu', 'infc_menu_actions');
$infc_limit = 100;

function infc_menu_actions() {
	add_submenu_page( 'edit-comments.php', 'Influential Commenters', 'Influential Commenters', 'manage_options', 'influential-commenters', 'infc_vista');
}

function infc_vista() {
	
	
	infc_header();
	
	infc_format_unreplied();
	
	infc_footer();
}

function infc_format_unreplied() {
	$commenters = infc_get_commenters();
	if (!empty($commenters)) {
		foreach ($commenters as $com) {
			$url = $com->comment_author_url;
			$alexa_rank = infc_get_alexa($url);
			$com->alexa = $alexa_rank;
		}
	}
	usort($commenters, 'infc_sort_by_alexa');
	?>
	<table class="widefat">
		<thead>
			<th>Author Name</th>
			<th>Author URL</th>
			<th>Author Email Address</th>
			<th>Author's Website Alexa Rank</th>
			<th># Facebook Shares</th>
			<th>Last Commented</th>
		</thead>
		<tbody>
	<?php
	global $infc_limit;
	$i = 0;
	if (!empty($commenters)) {
		foreach ($commenters as $com) {
			if ($i >= $infc_limit) {
				break;
			}
			$url = $com->comment_author_url;
			if ($com->alexa && $com->comment_author != 'Mr WordPress') {
				echo '<tr>';
				echo '<td>' . $com->comment_author . '</td>';
				echo '<td><a href="' . $url . '">' . $url . '</a></td>';
				echo '<td>' . $com->comment_author_email . '</td>';
				echo '<td>' . number_format(intval($com->alexa)) . '</td>';
				echo '<td>' . number_format(intval(infc_get_facebook_shares($url))) . '</td>';
				echo '<td>' . nicetime($com->comment_date) . '</td>';
				echo '</tr>';
			}
			$i++;
		}
	}
	?>
	</tbody>
	</table>
	<?php
}
function infc_sort_by_alexa($a, $b) {
    return $a->alexa - $b->alexa;
}

function infc_header() {
	$name = 'Influential Commenters';
	echo '<div class="infc_head">';
	echo '<h2>' . $name . '</h2>';
	echo '</div>';
	echo '<p>Shows your 100 most influential commenters in a list ordered by their website\'s Alexa rank.</p>';
}

function infc_footer(){
	echo '<script type="text/javascript">
	jQuery(document).ready(function ($) {
		$(".update-nag").hide();
	});
</script>';
}

function infc_get_alexa($url) {
	$response = simplexml_load_file('http://data.alexa.com/data?cli=10&dat=snbamz&url=' . $url);
	$pop = $response->SD[1]->POPULARITY;
	$rank = $pop['TEXT'];
	return $rank;
}

function infc_get_commenters() {
	    
    global $wpdb, $user_ID; // the Wordpress Database object, and the current user ID.

    $invalid_domains = array ( // no point including these
    	"'" . home_url() . "'", 
    	"'" . home_url() . "/'", 
    	"'http://www.facebook.com'", 
    	"'http://www.yahoo.com'", 
    	"'http://www.google.com'", 
    	"'http://www.gmail.com'",
    );
    $sql_invalids = implode(',', $invalid_domains);
    
    // We're getting the comments that the current user hasn't responded to yet.
    $ignored_comments = get_option('infc_ignored'); // this array will contain our ignored comments
    if (empty($ignored_comments)) {
    	$ignored_comments = array();
    }
    if ($wpdb && $user_ID) {
        
        $concat_sql = "SELECT comment_author_url, comment_date, comment_author_email, comment_author FROM $wpdb->comments WHERE comment_author_url NOT IN ($sql_invalids) GROUP BY comment_author_url ORDER BY comment_date";
       
        $commenters = $wpdb->get_results($concat_sql);
        
        return $commenters;
	}
}

function infc_get_facebook_shares($url) {
	$page = @file_get_contents('http://graph.facebook.com/' . $url);
	if (!empty($page)) {
		$data = json_decode($page);
	}
	$num_shares = $data->shares;
	return $num_shares;
}


function nicetime($date) {
	if (empty($date)) {
		return "No date provided";
	}

	$periods = array("second", "minute", "hour", "day", "week", "month", "year", "decade");
	$lengths  = array("60", "60", "24", "7", "4.35", "12", "10");

	$now             = time();
	$unix_date         = strtotime($date);

	// check validity of date
	if (empty($unix_date)) {
		return "Bad date";
	}

	// is it future date or past date
	if ($now > $unix_date) {
		$difference     = $now - $unix_date;
		$tense         = "ago";

	} else {
		$difference     = $unix_date - $now;
		$tense         = "from now";
	}

	for ($j = 0; $difference >= $lengths[$j] && $j < count($lengths)-1; $j++) {
		$difference /= $lengths[$j];
	}

	$difference = round($difference);

	if ($difference != 1) {
		$periods[$j].= "s";
	}

	$output = "$difference $periods[$j] {$tense}";
	if ($output == '0 seconds from now') {
		return 'A second ago';
	} else {
		return $output;
	}
}

?>